//
//  AgoraRtcEngine SDK
//
//  Copyright (c) 2019 Agora.io. All rights reserved.
//
#import <Foundation/Foundation.h>
#if TARGET_OS_OSX
#   import <Cocoa/Cocoa.h>
#else
#   import <UIKit/UIKit.h>
#   import <QuartzCore/QuartzCore.h>
#   define NSView UIView
#   define NSWindow UIWindow
#   define NSRect CGRect
#   define NSEvent UIEvent
#endif

/** The AgoraRDCEngineKit class is the entry point of the SDK providing API methods for apps to  easily start remote (desktop) controllability.
    
    NOTE: Get instance via AgoraRDCKit.sharedEngine with AgoraRDCConfig.
 */
@class AgoraRDCEngineKit;

/** `RDC_EVENT` describes the type of callbacks in engine working.
        
    @see AgoraRDCEngineDelegate for more information.
 */
typedef NS_ENUM(NSInteger, RDC_Event) {
    
    /* reserved */
    EVT_UNKNOWN,
    
    /* General errors reported
     * code - 8002 view boundary not set, call updateView first.
     */
    EVT_ERROR,
    
    /* reserved */
    EVT_WARN,
    
    /** RDC system login event callbacks.
     *  code - 0 success
     *       - otherwise error occurs, refer to:
     *  https://docs.agora.io/cn/Real-time-Messaging/API%20Reference/RTM_cpp/namespaceagora_1_1rtm.html#a41221aa09b01893e58f8e89bf947dd41
     */
    EVT_LOGIN,
    
    /** RDC system logout event callbacks.
     *  code - 0 success
     *       - otherwise error occurs, refer to:
     * https://docs.agora.io/cn/Real-time-Messaging/API%20Reference/RTM_cpp/namespaceagora_1_1rtm.html#af7a227d02fbb44647f30a4793f0eb79b
     */
    EVT_LOGOUT,
    
    /** RDC system join channel event callbacks.
     *  code - 0 success
     *       - otherwise error occurs, refer to:
     * https://docs.agora.io/cn/Real-time-Messaging/API%20Reference/RTM_cpp/namespaceagora_1_1rtm.html#adeb91ade364eddd742d8f3ad59c6638e
     */
    EVT_JOIN,
    
    /** RDC system leave channel event callbacks.
     *  code - 0 success
     *       - otherwise error occurs, refer to:
     * https://docs.agora.io/cn/Real-time-Messaging/API%20Reference/RTM_cpp/namespaceagora_1_1rtm.html#adeb91ade364eddd742d8f3ad59c6638e
     */
    EVT_LEAVE,
    
    /** RDC system connection state callbacks.
     *  code - 0 Uninitialize
     *       - 1 initialized
     *       - 2 Connected
     *       - 3 Disconnect
     */
    EVT_CONNECT_STATE,
    
    /** Callbacks when fail to send a message to peer side.
     * code - 0 success.
     *      - otherwise error occurs, refer to:
     * https://docs.agora.io/cn/Real-time-Messaging/API%20Reference/RTM_cpp/namespaceagora_1_1rtm.html#ac7c591aac4ca6867e239c8bcccc1fc5c
     */
    EVT_SEND_RESULT,
    
    
    /** Callbacks when `MASTER` uploads a file to server, it is a needed step of `paste to remote side`.
     * code - 0 success
     *      - otherwise error occurs, refer to:
     * https://docs.agora.io/cn/Real-time-Messaging/API%20Reference/RTM_cpp/namespaceagora_1_1rtm.html#a5fcd8b89f4b223f3aa8b01dea7d2747d
     */
    EVT_UPLOAD_RESULT,
    
    /** Callbacks when `SLAVE` downloads a file from server, it is a needed step of `paste to remote side`.
     * code - 0 success.
     *      - otherwise error occurs, refer to:
     * https://docs.agora.io/cn/Real-time-Messaging/API%20Reference/RTM_cpp/namespaceagora_1_1rtm.html#a26ce4ad61a9a9b7461548bb852723f9a
     */
    EVT_DOWNLOAD_RESULT,
    
    /** Notify current user when a file pasted from peer side.
     * code - 0 success
     * message - the full file path.
     */
    EVT_PASTED,
    
    /** Callbacks when `SLAVE` received a request from `MASTER` for controlling.
     * code - 0 success
     * message - the json data of request content.
     *         - for example: {"peer":"master","answer":false}
     */
    EVT_CONTROL_REQUEST,
    
    /** Callbacks when `SLAVE` received a request from `MASTER` for controlling.
     * code - 0 success
     * message - the json data of request content.
     *         - for example: {"peer":"master","answer":false}
     */
    EVT_CONTROL_QUIT,
    
    /** Notify current user when peer start to hold the control or not.
     * code - 0 success
     * message - {"hold": true | false }
     */
    EVT_HOLD_CONTROL,
    
    /** Notify master when cursor style changed on slave's side.
     * code - 0 success
     * message - {"cursor":[style],"x":x,"y":y,"peer":""}
     */
    EVT_MOUSE_STATE,
    
    /** Notify all audiences (NOT `MASTER` & `SLAVE`) for local renderer of mouse status.
     * code - 0 success
     * message - the json data of mouse state.
     *         - for example: {"cursor":0,"x":0,"y":0,"peer":"master"}
     */
    EVT_BROADCAST
}; // RDC_Event

/** Role's definition in scenario of remote control.
 *
 * @note `MASTER` is a controller sender and `SLAVE` is a receiver.
 *
 */
typedef NS_ENUM(NSInteger, Role) {
    Master = 0,
    Slave  = 1
}; // Role

/** The level definition of log filter. */
typedef NS_ENUM(NSInteger, LOG_LEVEL) {
    /* 0: (Default) Output logs of ALL levels. */
    LOG_TRACE = 0,
    
    /* 1: Output logs of the DEBUG, INFO, WARN, ERROR level. */
    LOG_DEBUG = 1,
    
    /* 2: Output logs of the INFO, WARN, ERROR level. */
    LOG_INFO  = 2,
    
    /* 3: Output logs of the WARN, ERROR level. */
    LOG_WARN  = 3,
    
    /* 4: Output logs of the ERROR level. */
    LOG_ERROR = 4
}; //LOG_LEVEL

/** Cursor types definition for local cursor renderring */
typedef NS_ENUM(NSInteger, CursorState) {
    ARROW        = 0,
    IBEAM        = 1,
    WAIT         = 2,
    CROSS        = 3,
    UPARROW      = 4,
    SIZENWSE     = 5,
    SIZENESW     = 6,
    SIZEWE       = 7,
    SIZENS       = 8,
    SIZEALL      = 9,
    FORBID       = 10,
    HAND         = 11,
    APPSTARTING  = 12,
    HELP         = 13
}; // CursorState

#pragma mark - AgoraRDCEngineDelegate
/** AgoraRDCEngineDelegate is call back from RDC SDK for events.
 *
 *  Application uses SDK should inherit this interface and receive all callbacks.
 *
 *  Sample usage:
 *   class MainController : AgoraRtcEngineDelegate {
 *     func onEvent(_ event: RDC_Event, code : Int, msg : String?) {
 *       switch event {
 *          case .EVT_LOGIN:
 *              ...
 *          case .EVT_LOGOUT:
 *              ...
 *       }
 *     }
 *   }
 */
@protocol AgoraRDCEngineDelegate <NSObject>

@required
- (void)onEvent:(RDC_Event)event code:(NSInteger)code msg:(NSString* _Nullable)msg;

@end

#pragma mark - AgoraRDConfig
/** The configuration of initialization of SDK.
 *
 *  @note Only way to pass into SDK is calling `AgoraRDCKit.sharedEngine `
 */
__attribute__((visibility("default"))) @interface AgoraRDConfig : NSObject

/* APP_ID of Agora RDC SDK (Required) */
@property (nonatomic, strong) NSString* _Nonnull appId;

/* Roles of SDK (Required) */
@property (assign, nonatomic) Role role;

/* Implementation of the callback interface should pass into SDK.(Required) */
@property (assign, nonatomic) id<AgoraRDCEngineDelegate> _Nullable delegate;

/** The directory of log file.
 *
 * @note (Default) NULL means the default system directory.
 *       [HomeDir]/Library/Logs/RDC
 */
@property (nonatomic, strong) NSString* _Nullable logPath;

/** The name of log file.
 *
 * @note (Default) NULL means `rdc.log`
 */
@property (nonatomic, strong) NSString* _Nullable logFileName;

/* Log filter of SDK */
@property (assign, nonatomic) LOG_LEVEL logLevel;

/** The max size of log file.
 *
 * @note (Default) 0 means infinite
 */
@property (assign, nonatomic) NSInteger logFileMaxSize;

/** The handle of a window component should be rendered by Remote Desktop graphics.
 *
 *  (Required)
 */
@property (assign, nonatomic) NSView* _Nonnull view;

@end

#pragma mark - AgoraRDCEngine

__attribute__((visibility("default"))) @interface AgoraRDCEngineKit : NSObject

/** Creates the AgoraRDCEngineKit object and returns the pointer. */
+ (instancetype _Nonnull)sharedEngineWithConfig:(AgoraRDConfig* _Nonnull)config;

/** Releases all `AgoraRDCEngineKit` resources including stopping the run loop.
 *
 * @return
 *    0 - success
 *    otherwise means error occurs
 */
- (void)destroy;

/** Logs in the Agora RDC system.
 *
 * @param[in] userId The user ID of the user logging in Agora's RDC system. The string must not exceed 64 bytes in length. It cannot be empty, null or "null".
 *  Supported characters:
 *  - All lowercase English letters: a to z
 *  - All uppercase English letters: A to Z
 *  - All numeric characters: 0 to 9
 *  - The space character.
 *  - Punctuation characters and other symbols, including: "!", "#", "$", "%", "&", "(", ")", "+", "-", ":", ";", "<", "=", ".", ">", "?", "@", "[", "]", "^", "_", " {", "}", "|", "~", ",
 * @param[in] Token used to log in the Agora RDC system and used when dynamic authentication is enabled.
 *
 * @return
 *    0 - success
 *    otherwise means error occurs
 */
- (int)login:(NSString* _Nonnull)userId token:(NSString* _Nullable)token;

/** Logs out of the Agora RDC system.
 *
 *  @return
 *    0 - success
 *    otherwise means error occurs
 */
- (int)logout;

/** Updates the area of control when the window of `MASTER` changed.
 *
 * @note ONLY works for `MASTER` role.
 *
 * Sample usages (MAC):
 *   let rect = window.frame
 *   let contentHeight = window.contentRect(forFrameRect: rect).size.height
 *   agoraRDCKit.updateViewPort( UInt(0), top: UInt(0), right: UInt(rect.width), bottom: UInt(contentHeight) )
 *
 * @param[in] left - left boundary of area.
 * @param[in] top  - top boundary of area.
 * @param[in] right - right boundary of area.
 * @param[in] bottom - bottom boundary of area.
 *
 * @return
 *    0 - success
 *    otherwise means error occurs.
 */
- (int)updateViewPort:(NSUInteger)left top:(NSUInteger)top right:(NSUInteger)right bottom:(NSUInteger)bottom;

/** Updates the monitor's view to share with `MASTER`
 *
 * @note ONLY works for `SLAVE` role.
 *
 * Sample usages:
 *   let screenId = 0 // 0 - primary screen, 1 - the secondary screen ...
 *   agoraRDCKit.updateView( screenId )
 *
 * @param[in] screenId - the index of screen, by default, primary screen is 0.
 *
 * @return
 *    0 - success
 *    otherwise means error occurs.
 */
- (int)updateView:(NSInteger)screenId;

/** Sends the request of control to `SLAVE` before the connection established. (optional)
 *
 *  Sample usages:
 *      agoraRDCKit.requestControl( peerId )
 *
 *  At the `SLAVE` side, the event `EVT_CONTROL_REQUEST` will be triggered.
 *    case EVT_CONTROL_REQUEST:
 *        agoraRDCKit.answerRequestControl( peerId, true ) // ACK for `MASTER` request.
 *
 * @param[in] peer - the user id logged in RDC system for `SLAVE`.
 *
 * @return
 *    0 - success
 *    otherwise means error occurs.
 */
- (int)requestControl:(NSString* _Nonnull)peer;

/** Sends the result of `requestControl` to the request side.
 *
 * @param[in] peer    - the user id logged in RDC system for `MASTER`.
 * @param[in] answer  - the agreement for controlling.
 *
 * @return
 *    0 - success
 *    otherwise means error occurs.
 */
- (int)answerRequestControl:(NSString* _Nonnull)peer answer:(BOOL)answer;

/** Sends the signal of quit control to the other (MASTER or SLAVE)
 *
 *  (Optional)
 *
 * Sample usages:
 *     agoraRDCKit.quitControl(peerId) //sends the quit signal to peer side.
 *
 * At peer side, an event `EVT_CONTROL_QUIT` will be triggered.
 *   case EVT_CONTROL_QUIT:
 *     // Do yourself's processing code.
 *
 * @param[in] peer - the user id logged in RDC system who you want to notice.
 *
 * @return
 *    0 - success
 *    otherwise means error occurs.
 */
- (int)quitControl:(NSString* _Nonnull)peer;

/** Sends the result of quit control to the request side.
 *
 *  (Optional)
 *
 * @param[in] peer - the user id logged in RDC system of request sender.
 * @param[in] answer  - the agreement for request.
 *
 * @return
 *    0 - success
 *    otherwise means error occurs.
 */
- (int)answerQuitControl:(NSString* _Nonnull)peer answer:(BOOL)answer;

/** Joins the specified channel.
 *
 * For others participants to watch remote desktop, you can use this method to join channel for sharing.
 *
 * Sample usages:
 *     agoraRDCKit.joinChannel("your channel id")
 *
 * An asynchronous event `EVT_JOIN` will be reported for join result.
 *   case EVT_JOIN:
 *     if (code == 0) { // success
 *       //adds your success handling codes here.
 *     } else {
 *       //adds your error handling codes here.
 *     }
 *
 * @param[in] channelId, channel id to join.
 *
 * @return
 *    0 - success
 *    otherwise means error occurs.
 */
- (int)joinChannel:(NSString* _Nonnull)channelId;

/** Leaves the specified channel.
 *
 * @return
 *    0 - success
 *    otherwise means error occurs.
 */
- (int)leaveChannel;

/** Notifies remote side when current window actived. (for the purpose of hide remote cursor)
 *
 * @return
 *    0 - success
 *    otherwise means error occurs
 */
- (int)holdControl:(BOOL)hold;

/** Executes operation to paste local file or text to remote side.
 *
 * @return
 *    0 - success
 *    otherwise means error occurs.
 */
- (int)remotePasteOp:(BOOL)isFile content:(NSString* _Nullable)content;

/** Asks for remote machine's information
 *
 * @return
 *    0 - success
 *    otherwise means error occurs.
 */
- (int)requestRemoteInfo:(NSString* _Nonnull)peer;
@end
