//
//  ScreenViewController.swift
//  RDC-MAC-Sample
//
//  Created by dellinger on 2022/1/6.
//

import Cocoa
import Carbon.HIToolbox
import AgoraRtcKit
import AgoraRDCKit

class ScreenViewController : BaseViewController, NSWindowDelegate {
    
    var channelId: String!
    var userName: String!
    var peer: String!
    var isMaster: Bool = false
    var agoraRtcKit: AgoraRtcEngineKit!
    var agoraRDCKit: AgoraRDCEngineKit!
    var streamId: Int = 0
    
    @IBOutlet weak var remoteView: NSView!
    @IBOutlet weak var remoteIndicator: NSImageView!
    @IBOutlet weak var txtMessage: NSTextField!
    
    struct ControlRequest : Codable {
        let peer : String
        let answer : Bool
        let msg : String
        
        enum CodingKeys: String, CodingKey {
            case peer
            case answer
            case msg
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        view.wantsLayer = true
        remoteView.wantsLayer = true
        
        agoraRtcKit = AgoraRtcEngineKit.sharedEngine(withAppId: AppID, delegate: self)
        agoraRtcKit.enableAudio()
        agoraRtcKit.enableVideo()
        if (isMaster) {
            txtMessage.stringValue = "Login ... "
            agoraRtcKit.muteLocalVideoStream(true)
        } else {
            txtMessage.stringValue = "Screen Sharing ... "
            agoraRtcKit.startScreenCapture(byDisplayId: UInt(CGMainDisplayID()),
                                           rectangle: CGRect.zero,
                                           parameters: AgoraScreenCaptureParameters())
        }
        
        let conf = AgoraVideoEncoderConfiguration(size: AgoraVideoDimension960x720,
                                                  frameRate: .fps15,
                                                  bitrate: AgoraVideoBitrateStandard,
                                                  orientationMode: .adaptative)
        agoraRtcKit.setVideoEncoderConfiguration(conf)
        
        let config = AgoraDataStreamConfig()
        let ret = agoraRtcKit.createDataStream(&streamId, config: config)
        if (ret != 0) {
            txtMessage.stringValue = "fail to create data stream, \(ret)"
            return
        }
        
        agoraRtcKit.joinChannel(byToken: Token, channelId: channelId, info: nil, uid: 0) { [self]
            (sid, uid, elapsed) -> Void
            in
            print("sid:\(sid), uid:\(uid), elapsed:\(elapsed)")
            self.loadRDCEngine()
        }
        
        NSEvent.addLocalMonitorForEvents(matching: .keyDown) {
            
            let shiftOn = $0.modifierFlags.contains(.shift)
            let ctrlOn = $0.modifierFlags.contains(.control)
            if (shiftOn && ctrlOn && kVK_ANSI_V == $0.keyCode) {
                self.agoraRDCKit.remotePasteOp(false, content: nil)
            }
            return $0
        }
    }
    
    override func viewWillAppear() {
        super.viewWillAppear()
        self.view.layer?.backgroundColor = NSColor.black.cgColor
        remoteView.layer?.backgroundColor = NSColor.clear.cgColor
        self.view.window?.delegate = self
    }
    
    override func viewDidAppear() {
        self.view.window?.title = channelId
    }
    
    override var representedObject: Any? {
        didSet {

        }
    }
    
    func showNotification(_ msg: String, title:String) {
        let notification = NSUserNotification()
        notification.title = title
        notification.informativeText = msg
        notification.soundName =    NSUserNotificationDefaultSoundName
        NSUserNotificationCenter.default.deliver(notification)
    }
    
    func windowDidBecomeKey(_ notification: Notification) {
        agoraRDCKit.holdControl(true)
    }
    
    func windowDidResignMain(_ notification: Notification) {
        agoraRDCKit.holdControl(false)
    }

    func windowShouldClose(_ sender: NSWindow) -> Bool {
        if confirm("Sure to exit?", .Warn) == .alertFirstButtonReturn {
            NSApp.terminate(self)
            return true
        } else {
            return false
        }
    }
    
    func windowDidResize(_ notification: Notification) {
        if (isMaster) {
            let rect = self.view.window!.frame
            let contentHeight = self.view.window?.contentRect(forFrameRect: rect).size.height
            let titleBarHeight = rect.size.height - contentHeight!
            let minX = UInt(0)
            let minY = UInt(0)
            let maxX = UInt(rect.width)
            let maxY = UInt(rect.height - titleBarHeight)
            agoraRDCKit.updateViewPort(minX, top: minY, right: maxX, bottom: maxY)
        }
    }
    
    func sendUserName() {
        let data = userName?.data(using: .utf8)
        let r = agoraRtcKit.sendStreamMessage(streamId, data: data!)
        if (r != 0) {
            txtMessage.stringValue = "send stream message fail, \(r)"
        }
    }
    
    func loadRDCEngine() {
        let logPath = NSSearchPathForDirectoriesInDomains(.cachesDirectory, .userDomainMask, true).first!
        
        let config = AgoraRDConfig()
        config.appId = AppID
        config.logPath = logPath
        config.logFileName = "rdc_mac.log"
        config.logFileMaxSize = 10 * 1024 * 1024
        config.role = isMaster ? .Master : .Slave
        config.delegate = self
        if isMaster {
            config.view = self.view
        }
        
        agoraRDCKit = AgoraRDCEngineKit.sharedEngine(with: config)
        let r = agoraRDCKit.login(userName, token: nil)
        if (r != 0) {
            alert("fail to login RDC system \(r)", .Error)
        }
        if (!isMaster) {
            agoraRDCKit.updateView(0)
        }
        
    }
}

extension ScreenViewController: AgoraRtcEngineDelegate {
   
    func rtcEngine(_ engine: AgoraRtcEngineKit, receiveStreamMessageFromUid uid: UInt, streamId: Int, data: Data) {
        let message = String.init(data: data, encoding: .utf8) ?? ""
        if !message.isEmpty {
            self.peer = message
            DispatchQueue.main.asyncAfter(deadline: .now() + 0.5) {
                self.agoraRDCKit.requestControl(self.peer)
            }
        }
    }

    func rtcEngine(_ engine: AgoraRtcEngineKit, didJoinedOfUid uid:UInt, elapsed:Int) {
        
        if (!isMaster) {
            sendUserName()
            return
        }
        
        if (remoteView.isHidden) {
            remoteView.isHidden = false
        }
        remoteIndicator.isHidden = true
        txtMessage.isHidden = true
        
        let videoCanvas = AgoraRtcVideoCanvas()
        videoCanvas.uid = uid
        videoCanvas.view = remoteView
        videoCanvas.renderMode = .fill
        agoraRtcKit.setupRemoteVideo(videoCanvas)
    }
    
    func rtcEngine(_ engine: AgoraRtcEngineKit, didOfflineOfUid uid:UInt, reason:AgoraUserOfflineReason) {
        remoteView.isHidden = true
        remoteIndicator.isHidden = false
        txtMessage.stringValue = "Peer offline, waiting ... "
        txtMessage.isHidden = false
        
    }
}

extension ScreenViewController: AgoraRDCEngineDelegate {
    func onEvent(_ event: RDC_Event, code: Int, msg: String?) {
        switch event {
        case .EVT_LOGIN:
            if code == 0 {
                DispatchQueue.main.async {
                    self.txtMessage.stringValue = "Login success"
                }
            } else {
                DispatchQueue.main.async {
                    self.txtMessage.stringValue = "Login fail, try later ... "
                }
                print("login fail \(code) - \(String(describing: msg))")
            }
            
        case .EVT_CONTROL_REQUEST:
            if (msg == nil) {
                return;
            }
            
            let decoder = JSONDecoder()
            let data = msg?.data(using: .utf8)!
            do {
                let req : ControlRequest = try decoder.decode(ControlRequest.self, from: data!)
                if req.answer {
                    if req.msg != "1" {
                        DispatchQueue.main.async {
                            self.showNotification("对方已拒绝控制", title: "RDC Demo v1.0")
                            self.alert("对方已拒绝控制", .Warn, "WARN")
                        }
                    }
                } else {
                    DispatchQueue.main.async {
                        self.showNotification("[" + req.peer + "]想控制您的设备，是否同意？", title: "RDC Demo v1.0")
                        if self.confirm("[" + req.peer + "]想控制您的设备，是否同意？", .Info) == .alertFirstButtonReturn {
                            self.agoraRDCKit.answerRequestControl(req.peer, answer: true)
                        } else {
                            self.agoraRDCKit.answerRequestControl(req.peer, answer: false)
                        }
                    }
                }
                
            } catch let ex {
                print(ex.localizedDescription)
            }
            break;
            
        case .EVT_HOLD_CONTROL:
            let param = AgoraScreenCaptureParameters()
            guard let message = msg else {
                return
            }
            if (message.contains("true")) {
                param.captureMouseCursor = false
            } else {
                param.captureMouseCursor = true
            }
            agoraRtcKit.update(param)
            break;
            
        case .EVT_MOUSE_STATE:
            guard let message = msg else {
                return
            }
            if let jsonData = message.data(using: .utf8) {

                let dicArr = try! JSONSerialization.jsonObject(with: jsonData, options: .allowFragments) as! [String:AnyObject]
                let v = dicArr["cursor"] as! NSNumber
                let cursor = AgoraRDCKit.CursorState(rawValue: v.intValue)
                switch cursor {
                case .ARROW:
                    NSCursor.arrow.set()
                    break
                case .IBEAM:
                    NSCursor.iBeam.set()
                    break
                case .WAIT:
                    NSCursor.operationNotAllowed.set()
                    break
                case .CROSS:
                    NSCursor.crosshair.set()
                    break
                case .UPARROW:
                    NSCursor.resizeUp.set()
                    break
                case .SIZENWSE:
                    NSCursor.resizeLeftRight.set()
                    break
                case .SIZENESW:
                    NSCursor.resizeUpDown.set()
                    break
                case .SIZEWE:
                    NSCursor.resizeLeftRight.set()
                    break
                case .SIZENS:
                    NSCursor.resizeUpDown.set()
                    break
                case .SIZEALL:
                    NSCursor.crosshair.set()
                    break
                case .FORBID:
                    NSCursor.operationNotAllowed.set()
                    break
                case .HAND:
                    NSCursor.pointingHand.set()
                    break
                case .APPSTARTING:
                    NSCursor.contextualMenu.set()
                    break
                case .HELP:
                    NSCursor.contextualMenu.set()
                    break
                default:
                    NSCursor.arrow.set()
                }
            }
            
            break;
            
        default:
            print("=== default")
        }
    }
}
